@@ -20,6 +20,12 @@
 #include "which.c"
 #include "compat.c"
 
+#include "safe_io.c"
+#include "xchdir.c"
+#include "xgetcwd.c"
+#include "xreadlink.c"
+#include "xsystem.c"
+
 /* custom libs */
 #include "atom_explode.c"
 #include "atom_compare.c"
