@@ -6,6 +6,7 @@ struct dirent *q_vdb_get_next_dir(DIR *dir)
 {
 	/* search for a category directory */
 	struct dirent *ret;
+	int len, i;
 
 next_entry:
 	ret = readdir(dir);
@@ -16,10 +17,21 @@ next_entry:
 
 	if (ret->d_name[0] == '.' || ret->d_name[0] == '-')
 		goto next_entry;
-	if (strchr(ret->d_name, '-') == NULL)
-		if ((strcmp(ret->d_name, "virtual")) != 0)
-			goto next_entry;
 
+	len = strlen(ret->d_name);
+	for (i = 0; i < len; i++) {
+		if (!isalnum(ret->d_name[i])) { /* [A-Za-z0-9+_.-] */
+			switch (ret->d_name[i]) {
+				case '+':
+				case '_':
+				case '.':
+				case '-':
+					break;
+				default:
+					goto next_entry;
+			}
+		}
+	}
 	return ret;
 }
 
