@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2008 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.163 2009/08/23 06:07:07 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.173 2010/01/17 00:25:35 vapier Exp $
  *
  * Copyright 2005-2008 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2008 Mike Frysinger  - <vapier@gentoo.org>
@@ -29,7 +29,6 @@
 #include <libgen.h>
 #include <limits.h>
 #include <assert.h>
-#include <dlfcn.h>
 #include "main.h"
 
 /* prototypes and such */
@@ -46,7 +45,6 @@ void cleanup(void);
 int lookup_applet_idx(const char *);
 
 /* variables to control runtime behavior */
-void *dlhandle = NULL;
 char *module_name = NULL;
 char *modpath = NULL;
 char exact = 0;
@@ -57,7 +55,7 @@ char pretend = 0;
 char reinitialize = 0;
 char reinitialize_metacache = 0;
 char portdir[_Q_PATH_MAX] = EPREFIX "/usr/portage";
-char portarch[20] = ""; 
+char portarch[20] = "";
 char portvdb[_Q_PATH_MAX] = "var/db/pkg";
 char portcachedir[] = "metadata/cache";
 char portroot[_Q_PATH_MAX] = "/";
@@ -120,7 +118,7 @@ void no_colors()
 	NULL
 #define COMMON_GETOPTS_CASES(applet) \
 	case 'v': ++verbose; break; \
-	case 'q': ++quiet; freopen("/dev/null", "w", stderr); break; \
+	case 'q': ++quiet; stderr = freopen("/dev/null", "w", stderr); break; \
 	case 'V': version_barf( applet ## _rcsid ); break; \
 	case 'h': applet ## _usage(EXIT_SUCCESS); break; \
 	case 'C': no_colors(); break; \
@@ -180,29 +178,31 @@ static void version_barf(const char *Id)
 
 static char eat_file(const char *file, char *buf, const size_t bufsize)
 {
-	FILE *f;
+	int fd;
 	struct stat s;
 	char ret = 0;
 
-	if ((f = fopen(file, "r")) == NULL)
+	if ((fd = open(file, O_RDONLY)) == -1)
 		return ret;
 
-	memset(buf, 0x00, bufsize);
-	if (fstat(fileno(f), &s) != 0)
+	buf[0] = '\0';
+	if (fstat(fd, &s) != 0)
 		goto close_and_ret;
 	if (s.st_size) {
 		if (bufsize < (size_t)s.st_size)
 			goto close_and_ret;
-		if (fread(buf, 1, s.st_size, f) != (size_t)s.st_size)
+		if (read(fd, buf, s.st_size) != (ssize_t)s.st_size)
 			goto close_and_ret;
+		buf[s.st_size] = '\0';
 	} else {
-		if (fread(buf, 1, bufsize, f) == 0)
+		if (read(fd, buf, bufsize) == 0)
 			goto close_and_ret;
+		buf[bufsize] = '\0';
 	}
 
 	ret = 1;
 close_and_ret:
-	fclose(f);
+	close(fd);
 	return ret;
 }
 
@@ -475,7 +475,7 @@ void initialize_portage_env(void)
 	struct stat st;
 	FILE *fp;
 	char buf[BUFSIZE], *s, *p;
-	char *e = EPREFIX;
+	char *e = (char *) EPREFIX;
 
 	char profile[_Q_PATH_MAX], portage_file[_Q_PATH_MAX];
 	const char *files[] = {portage_file, EPREFIX "/etc/make.globals", EPREFIX "/etc/make.conf"};
@@ -760,7 +760,7 @@ ret:
 
 void reinitialize_ebuild_flat(void)
 {
-	if ((chdir(portdir)) != 0) {
+	if (chdir(portdir) != 0) {
 		warnp("chdir to PORTDIR '%s' failed", portdir);
 		return;
 	}
@@ -814,59 +814,6 @@ portage_cache *cache_read_file(const char *file)
 		goto err;
 	len = sizeof(*ret) + s.st_size + 1;
 	ret = xzalloc(len);
-#if 0
-{	/* This is the expected format of portage-2.0 cache code NAME=var */
-	/* at any time ferringb might decide to add new entries. CDEPEND= will be no more */
-	char buf[BUFSIZ];
-	while ((fgets(buf, sizeof(buf), f)) != NULL) {
-		if ((ptr = strrchr(buf, '\n')) != NULL)
-			*ptr = 0;
-
-		if ((strncmp(buf, "DEPEND=", 7)) == 0)
-			ret->DEPEND = xstrdup(buf + 7);
-
-		if ((strncmp(buf, "DESCRIPTION=", 12)) == 0)
-			ret->DESCRIPTION = xstrdup(buf + 12);
-#if 0
-		if ((strncmp(buf, "EAPI=", 5)) == 0)
-			warnf("EAPI=%s", buf + 5);
-#endif
-		if ((strncmp(buf, "HOMEPAGE=", 9)) == 0)
-			ret->HOMEPAGE = xstrdup(buf + 9);
-
-		if ((strncmp(buf, "INHERITED=", 10)) == 0)
-			ret->INHERITED = xstrdup(buf + 10);
-
-		if ((strncmp(buf, "IUSE=", 4)) == 0)
-			ret->IUSE = xstrdup(buf + 4);
-
-		if ((strncmp(buf, "KEYWORDS=", 9)) == 0)
-			ret->KEYWORDS = xstrdup(buf + 9);
-
-		if ((strncmp(buf, "LICENSE=", 8)) == 0)
-			ret->LICENSE = xstrdup(buf + 8);
-
-		if ((strncmp(buf, "PDEPEND=", 8)) == 0)
-			ret->PDEPEND = xstrdup(buf + 8);
-
-		if ((strncmp(buf, "PROVIDE=", 8)) == 0)
-			ret->PROVIDE = xstrdup(buf + 8);
-
-		if ((strncmp(buf, "RDEPEND=", 8)) == 0)
-			ret->RDEPEND = xstrdup(buf + 8);
-
-		if ((strncmp(buf, "RESTRICT=", 9)) == 0)
-			ret->RESTRICT = xstrdup(buf + 9);
-
-		if ((strncmp(buf, "SLOT=", 5)) == 0)
-			ret->SLOT = xstrdup(buf + 5);
-
-		if ((strncmp(buf, "SRC_URI=", 8)) == 0)
-			ret->SRC_URI = xstrdup(buf + 8);
-	}
-	rewind(f);
-}
-#endif
 	ptr = (char*)ret;
 	ret->_data = ptr + sizeof(*ret);
 	if ((off_t)fread(ret->_data, 1, s.st_size, f) != s.st_size)
@@ -961,7 +908,8 @@ char *grab_vdb_item(const char *item, const char *CATEGORY, const char *PF)
 	snprintf(buf, sizeof(buf), "%s%s/%s/%s/%s", portroot, portvdb, CATEGORY, PF, item);
 	if ((fp = fopen(buf, "r")) == NULL)
 		return NULL;
-	fgets(buf, sizeof(buf), fp);
+	if (fgets(buf, sizeof(buf), fp) == NULL)
+		buf[0] = '\0';
 	if ((p = strchr(buf, '\n')) != NULL)
 		*p = 0;
 	fclose(fp);
@@ -986,9 +934,9 @@ queue *get_vdb_atoms(int fullcpv)
 	depend_atom *atom = NULL;
 	queue *cpf = NULL;
 
-	getcwd(savecwd, sizeof(savecwd));
+	xgetcwd(savecwd, sizeof(savecwd));
 
-	assert(chdir(savecwd) == 0);
+	xchdir(savecwd);
 
 	if (chdir(portroot) != 0)
 		goto fuckit;
@@ -1007,7 +955,7 @@ queue *get_vdb_atoms(int fullcpv)
 		if (chdir(cat[j]->d_name) != 0)
 			continue;
 		if ((dfd = scandir(".", &pf, filter_hidden, alphasort)) < 0) {
-			chdir("..");
+			xchdir("..");
 			continue;
 		}
 		for (i = 0; i < dfd; i++) {
@@ -1038,7 +986,7 @@ queue *get_vdb_atoms(int fullcpv)
 			atom_implode(atom);
 			cpf = add_set(buf, slot, cpf);
 		}
-		chdir("..");
+		xchdir("..");
 		while (dfd--) free(pf[dfd]);
 		free(pf);
 	}
@@ -1048,7 +996,7 @@ queue *get_vdb_atoms(int fullcpv)
 	free(cat);
 
 fuckit:
-	assert(chdir(savecwd) == 0);
+	xchdir(savecwd);
 	return cpf;
 }
 
