@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qfile.c,v 1.45 2007/05/24 14:47:18 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qfile.c,v 1.51 2010/01/16 23:45:02 vapier Exp $
  *
  * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
@@ -13,7 +13,7 @@
 #define QFILE_DEFAULT_MAX_ARGS 5000
 #define QFILE_DEFAULT_MAX_ARGS_STR "5000"
 
-#define QFILE_FLAGS "ef:m:oRx:" COMMON_FLAGS
+#define QFILE_FLAGS "ef:m:oRx:S" COMMON_FLAGS
 static struct option const qfile_long_opts[] = {
 	{"exact",       no_argument, NULL, 'e'},
 	{"from",        a_argument,  NULL, 'f'},
@@ -21,6 +21,7 @@ static struct option const qfile_long_opts[] = {
 	{"orphans",     no_argument, NULL, 'o'},
 	{"root-prefix", no_argument, NULL, 'R'},
 	{"exclude",     a_argument,  NULL, 'x'},
+	{"slots",       no_argument, NULL, 'S'},
 	COMMON_LONG_OPTS
 };
 static const char *qfile_opts_help[] = {
@@ -30,9 +31,10 @@ static const char *qfile_opts_help[] = {
 	"List orphan files",
 	"Assume arguments are already prefixed by $ROOT",
 	"Don't look in package <arg>",
+	"Display installed packages with slots",
 	COMMON_OPTS_HELP
 };
-static char qfile_rcsid[] = "$Id: qfile.c,v 1.45 2007/05/24 14:47:18 solar Exp $";
+static char qfile_rcsid[] = "$Id: qfile.c,v 1.51 2010/01/16 23:45:02 vapier Exp $";
 #define qfile_usage(ret) usage(ret, QFILE_FLAGS, qfile_long_opts, qfile_opts_help, lookup_applet_idx("qfile"))
 
 #define qfile_is_prefix(path, prefix, prefix_length) \
@@ -53,6 +55,12 @@ typedef struct {
 	char *exclude_slot;
 } qfile_args_t;
 
+char slotted = 0;
+
+/*
+ * We assume the people calling us have chdir(/var/db/pkg) and so
+ * we use relative paths throughout here.
+ */
 void qfile(char *, const char *, qfile_args_t *);
 void qfile(char *path, const char *root, qfile_args_t *args)
 {
@@ -60,11 +68,10 @@ void qfile(char *path, const char *root, qfile_args_t *args)
 	DIR *dir;
 	struct dirent *dentry;
 	char *entry_basename;
-	char *entry_dirname;
 	char *p;
 	char buf[1024];
-	char pkg[126];
-	depend_atom *atom;
+	char pkg[150];
+	depend_atom *atom = NULL;
 	int i, path_ok;
 	char bn_firstchar;
 	char *real_root = args->real_root;
@@ -74,15 +81,29 @@ void qfile(char *path, const char *root, qfile_args_t *args)
 	char *bn_firstchars = args->bn_firstchars;
 	short *non_orphans = args->non_orphans;
 
-	if (chdir(path) != 0 || (dir = opendir(".")) == NULL)
+	if ((dir = opendir(path)) == NULL) {
+		warnp("opendir(%s) failed", path);
 		return;
+	}
 
 	while ((dentry = readdir(dir))) {
 		if (dentry->d_name[0] == '.')
 			continue;
 
-		snprintf(pkg, sizeof(pkg), "%s/%s", basename(path), dentry->d_name);
-		atom = NULL; /* Will be exploded once at most, as needed. */
+		/* We reuse pkg for reading files in the subdir */
+		p = basename(path);
+		i = snprintf(pkg, sizeof(pkg), "%s/%s", p, dentry->d_name);
+		if (i + 20 >= sizeof(pkg)) {
+			warn("skipping long pkg name: %s/%s", p, dentry->d_name);
+			continue;
+		}
+		p = pkg + i;
+
+		/* Will be exploded once at most, as needed. */
+		if (atom) {
+			atom_implode(atom);
+			atom = NULL;
+		}
 
 		/* If exclude_pkg is not NULL, check it.  We are looking for files
 		 * collisions, and must exclude one package.
@@ -102,38 +123,23 @@ void qfile(char *path, const char *root, qfile_args_t *args)
 					&& strcmp(args->exclude_pkg, atom->PN) != 0)
 				goto dont_skip_pkg; /* "(CAT/)?PN" doesn't match */
 check_pkg_slot: /* Also compare slots, if any was specified */
-			if (args->exclude_slot == NULL) {
-				if (atom != NULL)
-					atom_implode(atom);
+			if (args->exclude_slot == NULL)
 				continue; /* "(CAT/)?(PN|PF)" matches, and no SLOT specified */
-			}
 			buf[0] = '0'; buf[1] = '\0';
-			xasprintf(&p, "%s/%s/SLOT", path, dentry->d_name);
-			if ((fp = fopen(p, "r")) != NULL) {
-				free(p);
-				if (fgets(buf, sizeof(buf), fp) != NULL)
-					if ((p = strchr(buf, '\n')) != NULL)
-						*p = 0;
-				fclose(fp);
-			} else {
-				free(p);
-			}
-			if (strncmp(args->exclude_slot, buf, sizeof(buf)) == 0) {
-				if (atom != NULL)
-					atom_implode(atom);
+			strcpy(p, "/SLOT");
+			eat_file(pkg, buf, sizeof(buf));
+			rmspace(buf);
+			*p = '\0';
+			if (strncmp(args->exclude_slot, buf, sizeof(buf)) == 0)
 				continue; /* "(CAT/)?(PN|PF):SLOT" matches */
-			}
 		}
 dont_skip_pkg: /* End of the package exclusion tests. */
 
-		xasprintf(&p, "%s/%s/CONTENTS", path, dentry->d_name);
-		if ((fp = fopen(p, "r")) == NULL) {
-			free(p);
-			if (atom != NULL)
-				atom_implode(atom);
+		strcpy(p, "/CONTENTS");
+		fp = fopen(pkg, "r");
+		*p = '\0';
+		if (fp == NULL)
 			continue;
-		}
-		free(p);
 
 		while ((fgets(buf, sizeof(buf), fp)) != NULL) {
 			contents_entry *e;
@@ -141,10 +147,9 @@ dont_skip_pkg: /* End of the package exclusion tests. */
 			if (!e)
 				continue;
 
-			/* much faster than using basename(), since no need to strdup */
-			if ((entry_basename = strrchr(e->name, '/')) == NULL)
+			/* assume sane basename() -- doesnt modify argument */
+			if ((entry_basename = basename(e->name)) == NULL)
 				continue;
-			entry_basename++;
 
 			/* used to cut the number of strcmp() calls */
 			bn_firstchar = entry_basename[0];
@@ -162,70 +167,79 @@ dont_skip_pkg: /* End of the package exclusion tests. */
 
 				if (!path_ok) {
 					/* check the full filepath ... */
-					entry_dirname = xstrdup(e->name);
-					if ((p = strrchr(entry_dirname, '/')) == NULL) {
-						free(entry_dirname);
-						continue;
-					}
-					if (p == entry_dirname)
-						/* (e->name == "/foo") ==> dirname == "/" */
-						*(p + 1) = '\0';
-					else
-						*p = '\0';
-					if (dir_names[i] != NULL &&
-							strcmp(entry_dirname, dir_names[i]) == 0)
+					size_t dirname_len = (entry_basename - e->name - 1);
+					/* basename(/usr)     = usr, dirname(/usr)     = /
+					 * basename(/usr/bin) = bin, dirname(/usr/bin) = /usr
+					 */
+					if (dirname_len == 0)
+						dirname_len = 1;
+
+					if (dir_names[i] &&
+					    strncmp(e->name, dir_names[i], dirname_len) == 0 &&
+					    dir_names[i][dirname_len] == '\0')
 						/* dir_name == dirname(CONTENTS) */
 						path_ok = 1;
-					else if (real_dir_names[i] != NULL &&
-							strcmp(entry_dirname, real_dir_names[i]) == 0)
+
+					else if (real_dir_names[i] &&
+					         strncmp(e->name, real_dir_names[i], dirname_len) == 0 &&
+					         real_dir_names[i][dirname_len] == '\0')
 						/* real_dir_name == dirname(CONTENTS) */
 						path_ok = 1;
-					else {
-						char rpath[_Q_PATH_MAX+1];
-						char *fullpath = entry_dirname;
-						errno = 0;
-						if (real_root != NULL && real_root[0] != '\0')
-							xasprintf(&fullpath, "%s%s", real_root, entry_dirname);
-						realpath(fullpath, rpath);
-						if (errno != 0) {
+
+					else if (real_root[0]) {
+						char rpath[_Q_PATH_MAX + 1], *_rpath;
+						char *fullpath;
+						size_t real_root_len = strlen(real_root);
+
+						xasprintf(&fullpath, "%s%s", real_root, e->name);
+						fullpath[real_root_len + dirname_len] = '\0';
+						_rpath = rpath + real_root_len;
+						if (realpath(fullpath, rpath) == NULL) {
 							if (verbose) {
 								warnp("Could not read real path of \"%s\" (from %s)", fullpath, pkg);
-								warn("We'll never know whether \"%s/%s\" was a result for your query...",
-										entry_dirname, entry_basename);
+								warn("We'll never know whether \"%s\" was a result for your query...",
+										e->name);
 							}
-						} else if (!qfile_is_prefix(rpath, real_root, strlen(real_root))) {
+						} else if (!qfile_is_prefix(rpath, real_root, real_root_len)) {
 							if (verbose)
 								warn("Real path of \"%s\" is not under ROOT: %s", fullpath, rpath);
-						} else if (dir_names[i] != NULL &&
-								strcmp(rpath + strlen(real_root), dir_names[i]) == 0)
+						} else if (dir_names[i] &&
+						           strcmp(_rpath, dir_names[i]) == 0) {
 							/* dir_name == realpath(dirname(CONTENTS)) */
 							path_ok = 1;
-						else if (real_dir_names[i] != NULL &&
-								strcmp(rpath + strlen(real_root), real_dir_names[i]) == 0)
+						} else if (real_dir_names[i] &&
+						           strcmp(_rpath, real_dir_names[i]) == 0) {
 							/* real_dir_name == realpath(dirname(CONTENTS)) */
 							path_ok = 1;
-						if (fullpath != entry_dirname)
-							free(fullpath);
+						}
+						free(fullpath);
 					}
-					free(entry_dirname);
 				}
 				if (!path_ok)
 					continue;
 
 				if (non_orphans == NULL) {
+					char pkgslot[126];
+
 					if (atom == NULL && (atom = atom_explode(pkg)) == NULL) {
 						warn("invalid atom %s", pkg);
 						continue;
 					}
-
-					printf("%s%s/%s%s%s", BOLD, atom->CATEGORY, BLUE,
-						(exact ? dentry->d_name : atom->PN), NORM);
+					if (slotted) {
+						strcpy(p, "/SLOT");
+						eat_file(pkg, pkgslot+1, sizeof(pkgslot)-1);
+						rmspace(pkgslot+1);
+						*pkgslot = ':';
+						*p = '\0';
+					} else
+						*pkgslot = '\0';
+					printf("%s%s/%s%s%s%s", BOLD, atom->CATEGORY, BLUE,
+						(exact ? dentry->d_name : atom->PN),
+						pkgslot, NORM);
 					if (quiet)
 						puts("");
-					else if (root != NULL)
-						printf(" (%s%s)\n", root, e->name);
 					else
-						printf(" (%s)\n", e->name);
+						printf(" (%s%s)\n", root ? : "", e->name);
 
 				} else {
 					non_orphans[i] = 1;
@@ -234,8 +248,6 @@ dont_skip_pkg: /* End of the package exclusion tests. */
 			}
 		}
 		fclose(fp);
-		if (atom != NULL)
-			atom_implode(atom);
 	}
 	closedir(dir);
 
@@ -326,9 +338,7 @@ int prepare_qfile_args(const int argc, const char **argv,
 		warn("Could not get absolute path for ROOT (\"%s\"), because of missing or not absolute $PWD", tmppath);
 		return -1;
 	}
-	errno = 0;
-	realpath(tmppath, abspath);
-	if (errno != 0) {
+	if (realpath(tmppath, abspath) == NULL) {
 		free(pwd);
 		warnp("Could not read real path of ROOT (\"%s\")", tmppath);
 		return -1;
@@ -394,9 +404,7 @@ int prepare_qfile_args(const int argc, const char **argv,
 			if (abspath[real_root_length] == '\0')
 				strncat(abspath, "/", 1);
 			dirnames[i] = xstrdup(abspath + real_root_length);
-			errno = 0;
-			realpath(abspath, tmppath);
-			if (errno != 0) {
+			if (realpath(abspath, tmppath) == NULL) {
 				if (verbose) {
 					warnp("Could not read real path of \"%s\"", abspath);
 					warn("Results for query item \"%s\" may be inaccurate.", argv[i]);
@@ -416,9 +424,7 @@ int prepare_qfile_args(const int argc, const char **argv,
 			 * Dirname is meaningless here, we can only get realpath of the full
 			 * path and then split it.
 			 */
-			errno = 0;
-			realpath(abspath, tmppath);
-			if (errno != 0) {
+			if (realpath(abspath, tmppath) == NULL) {
 				warnp("Could not read real path of \"%s\"", abspath);
 				goto skip_query_item;
 			}
@@ -494,6 +500,7 @@ int qfile_main(int argc, char **argv)
 	while ((i = GETOPT_LONG(QFILE, qfile, "")) != -1) {
 		switch (i) {
 			COMMON_GETOPTS_CASES(qfile)
+			case 'S': slotted = 1; break;
 			case 'e': exact = 1; break;
 			case 'f':
 				if (args_file != NULL) {
@@ -540,15 +547,8 @@ int qfile_main(int argc, char **argv)
 	if ((args_file == NULL) && (max_args != QFILE_DEFAULT_MAX_ARGS))
 		warn("--max-args is only used when reading arguments from a file (with -f)");
 
-	if (chdir(portroot)) {
-		warnp("could not chdir(%s) for ROOT", portroot);
-		goto exit;
-	}
-
-	if (chdir(portvdb) != 0) {
-		warnp("could not chdir(ROOT/%s) for installed packages database", portvdb);
-		goto exit;
-	}
+	xchdir(portroot);
+	xchdir(portvdb);
 
 	/* Get a copy of $ROOT, with no trailing slash
 	 * (this one is just for qfile(...) output)
@@ -597,10 +597,8 @@ int qfile_main(int argc, char **argv)
 		if (nb_of_queries < 0)
 			goto exit;
 
-		if (chdir(portroot)
-				|| chdir(portvdb) != 0
-				|| (dir = opendir(".")) == NULL) {
-			warnp("could not chdir(ROOT/%s) for installed packages database", portvdb);
+		if ((dir = opendir(".")) == NULL) {
+			warnp("could not read(ROOT/%s) for installed packages database", portvdb);
 			goto exit;
 		}
 
