@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.83 2009/05/03 17:19:13 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.92 2010/01/13 19:16:22 vapier Exp $
  *
  * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
@@ -55,7 +55,7 @@ static const char *qmerge_opts_help[] = {
 	COMMON_OPTS_HELP
 };
 
-static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.83 2009/05/03 17:19:13 solar Exp $";
+static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.92 2010/01/13 19:16:22 vapier Exp $";
 #define qmerge_usage(ret) usage(ret, QMERGE_FLAGS, qmerge_long_opts, qmerge_opts_help, lookup_applet_idx("qmerge"))
 
 char search_pkgs = 0;
@@ -197,7 +197,7 @@ void fetch(const char *destdir, const char *src)
 		snprintf(buf, sizeof(buf), "%s " BUSYBOX " wget %s -P %s %s/%s", (force_download || install) ? "" : pretend ? "echo " : "",
 			(quiet ? "-q" : ""), destdir, binhost, src);
 	}
-	system(buf);
+	xsystem(buf);
 	fflush(stdout);
 	fflush(stderr);
 }
@@ -223,10 +223,8 @@ void qmerge_initialize(const char *Packages)
 		mkdir(port_tmpdir, 0755);
 	}
 
-	if (chdir(port_tmpdir) != 0)
-		errf("!!! chdir(PORTAGE_TMPDIR %s) %s", port_tmpdir, strerror(errno));
-	if (chdir("portage") != 0)
-		errf("!!! chdir(%s/portage) %s", port_tmpdir, strerror(errno));
+	xchdir(port_tmpdir);
+	xchdir("portage");
 
 	if (force_download && force_download != 2)
 		unlink(Packages);
@@ -296,7 +294,7 @@ void crossmount_rm(char *buf, const size_t size, const char *fname, const struct
 	}
 	qprintf("%s<<<%s %s\n", YELLOW, NORM, buf);
 	snprintf(buf, size, BUSYBOX " rm -rf ./%s", fname);
-	system(buf);
+	xsystem(buf);
 }
 
 void install_mask_pwd(int argc, char **argv, const struct stat st);
@@ -527,19 +525,20 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 	if (pretend)
 		return;
 
-	if (chdir(port_tmpdir) != 0) errf("!!! chdir(port_tmpdir %s) %s", port_tmpdir, strerror(errno));
+	xchdir(port_tmpdir);
 
 	mkdir(pkg->PF, 0755);
 	/* mkdir(pkg->PF, 0710); */
-	if (chdir(pkg->PF) != 0) errf("!!! chdir(PF %s) %s", pkg->PF, strerror(errno));
+	xchdir(pkg->PF);
 
-	system(BUSYBOX " rm -rf ./*"); /* this line does funny things to nano's highlighting. */
+	xsystem(BUSYBOX " rm -rf ./*"); /* this line does funny things to nano's highlighting. */
 
 	/* split the tbz and xpak data */
 	snprintf(tarball, sizeof(tarball), "%s.tbz2", pkg->PF);
 	snprintf(buf, sizeof(buf), "%s/%s/%s", pkgdir, pkg->CATEGORY, tarball);
 	unlink(tarball);
-	symlink(buf, tarball);
+	if (symlink(buf, tarball))
+		errp("symlink(%s, %s) failed", buf, tarball);
 	mkdir("vdb", 0755);
 	mkdir("image", 0755);
 
@@ -559,11 +558,11 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 	/* list and extract vdb files from the xpak */
 	snprintf(buf, sizeof(buf), "qxpak -d %s/%s/vdb -x %s.xpak `qxpak -l %s.xpak`",
 		port_tmpdir, pkg->PF, pkg->PF, pkg->PF);
-	system(buf);
+	xsystem(buf);
 
 	/* extrct the binary package data */
 	snprintf(buf, sizeof(buf), BUSYBOX " tar -jx%sf %s.tar.bz2 -C image/", ((verbose > 1) ? "v" : ""), pkg->PF);
-	system(buf);
+	xsystem(buf);
 	fflush(stdout);
 
 	/* check for an already installed pkg */
@@ -603,7 +602,7 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 		freeargv(ARGC, ARGV);
 		ARGC = 0; ARGV = NULL;
 	}
-	assert(chdir("image") == 0);
+	xchdir("image");
 
 	if (stat("./", &st) == (-1))
 		err("Cant stat pwd");
@@ -611,9 +610,9 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 	makeargv(install_mask, &iargc, &iargv);
 	install_mask_pwd(iargc, iargv, st);
 
-	if ((strstr(features, "noinfo")) != NULL) if (access("./usr/share/info", R_OK) == 0) system(BUSYBOX " rm -rf ./usr/share/info");
-	if ((strstr(features, "noman"))  != NULL) if (access("./usr/share/man",  R_OK) == 0) system(BUSYBOX " rm -rf ./usr/share/man");
-	if ((strstr(features, "nodoc"))  != NULL) if (access("./usr/share/doc",  R_OK) == 0) system(BUSYBOX " rm -rf ./usr/share/doc");
+	if ((strstr(features, "noinfo")) != NULL) if (access("./usr/share/info", R_OK) == 0) xsystem(BUSYBOX " rm -rf ./usr/share/info");
+	if ((strstr(features, "noman"))  != NULL) if (access("./usr/share/man",  R_OK) == 0) xsystem(BUSYBOX " rm -rf ./usr/share/man");
+	if ((strstr(features, "nodoc"))  != NULL) if (access("./usr/share/doc",  R_OK) == 0) xsystem(BUSYBOX " rm -rf ./usr/share/doc");
 
 	/* we dont care about the return code */
 	rmdir("./usr/share");
@@ -667,7 +666,7 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 		/* syntax: dir dirname */
 		if (S_ISDIR(st.st_mode)) {
 			mkdir(dest, st.st_mode);
-			chown(dest, st.st_uid, st.st_gid);
+			assert(chown(dest, st.st_uid, st.st_gid) == 0);
 			snprintf(line, sizeof(line), "dir %s", &buf[1]);
 		}
 
@@ -689,7 +688,7 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 					qprintf("%s***%s %s\n", BRYELLOW, NORM, &buf[1]);
 					if (verbose) {
 						snprintf(newbuf, sizeof(newbuf), "diff -u %s %s", buf, dest);
-						system(newbuf);
+						xsystem(newbuf);
 					}
 					continue;
 				}
@@ -728,7 +727,7 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 			memset(&path, 0, sizeof(path));
 
 			assert(strlen(dest));
-			readlink(buf, path, sizeof(path));
+			xreadlink(buf, path, sizeof(path) - 1);
 			assert(strlen(path));;
 
 			snprintf(line, sizeof(line), "sym %s -> %s%s%lu", &buf[1], path, " ", st.st_mtime);
@@ -737,18 +736,17 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 			strcpy(tmp, &buf[1]);
 			if (tmp[0] != '/') errf("sym does not start with /");
 
-			getcwd(pwd, sizeof(pwd));
-			if (chdir(dirname(tmp)) != 0) /* tmp gets eatten up now by the dirname call */
-				errf("chdir to symbolic dirname %s: %s", tmp, strerror(errno));
+			xgetcwd(pwd, sizeof(pwd));
+			xchdir(dirname(tmp)); /* tmp gets eatten up now by the dirname call */
 			if (lstat(path, &lst) != (-1))
 				unlink_q(dest);
 			/* if (path[0] != '/')
 				puts("path does not start with /");
 			*/
-			if ((symlink(path, dest)) != 0)
+			if (symlink(path, dest))
 				if (errno != EEXIST)
-					warn("symlink failed %s -> %s: %s", path, &buf[1], strerror(errno));
-			chdir(pwd);
+					warnp("symlink failed %s -> %s", path, &buf[1]);
+			xchdir(pwd);
 		}
 		/* Save the line to the contents file */
 		if (*line) fprintf(contents, "%s\n", line);
@@ -763,13 +761,13 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 	fclose(contents);
 	pclose(fp);
 
-	if (chdir(port_tmpdir) != 0) errf("!!! chdir(%s) %s", port_tmpdir, strerror(errno));
-	if (chdir(pkg->PF) != 0) errf("!!! chdir(%s) %s", pkg->PF, strerror(errno));
+	xchdir(port_tmpdir);
+	xchdir(pkg->PF);
 
 	snprintf(buf, sizeof(buf), "%s/var/db/pkg/%s/", portroot, pkg->CATEGORY);
 	if (access(buf, R_OK|W_OK|X_OK) != 0)
 		mkdirhier(buf, 0755);
-	strncat(buf, pkg->PF, sizeof(buf));
+	strncat(buf, pkg->PF, sizeof(buf)-strlen(buf)-1);
 
 	/* FIXME */ /* move unmerging to around here ? */
 	/* not perfect when a version is already installed */
@@ -777,7 +775,7 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 		char buf2[sizeof(buf)] = "";
 		/* we need to compare CONTENTS in it and remove any file not provided by our CONTENTS */
 		snprintf(buf2, sizeof(buf2), BUSYBOX " rm -rf %s", buf);
-		system(buf2);
+		xsystem(buf2);
 	}
 	if ((fp = fopen("vdb/COUNTER", "w")) != NULL) {
 		fputs("0", fp);
@@ -789,15 +787,15 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 	if (which("ebuild") != NULL) {
 		char *tbuf;
 		xasprintf(&tbuf, "ebuild %s/%s.ebuild postinst", buf, basename(buf));
-		system(tbuf);
+		xsystem(tbuf);
 		free(tbuf);
 	}
 
 	snprintf(buf, sizeof(buf), BUSYBOX " %s.tar.bz2", pkg->PF);
 	unlink_q(buf);
-	chdir(port_tmpdir);
+	xchdir(port_tmpdir);
 	snprintf(buf, sizeof(buf), "rm -rf %s", pkg->PF);
-	system(buf);
+	xsystem(buf);
 
 	snprintf(buf, sizeof(buf), "%s/%s.tbz2", pkgdir, pkg->PF);
 	unlink(buf);
@@ -867,7 +865,7 @@ int pkg_unmerge(char *cat, char *pkgname)
 							break;
 						}
 					} else {
-						warn("lstat failed for %s -> '%s': %s", dst, e->sym_target, strerror(errno));
+						warnp("lstat failed for %s -> '%s'", dst, e->sym_target);
 					}
 					warn("!!! %s -> %s", e->name, e->sym_target);
 					break;
@@ -880,7 +878,7 @@ int pkg_unmerge(char *cat, char *pkgname)
 						warn("%s is not a symlink", dst);
 					}
 				} else {
-					warn("lstat failed for '%s' -> %s: %s", dst, e->sym_target, strerror(errno));
+					warnp("lstat failed for '%s' -> %s", dst, e->sym_target);
 				}
 				break;
 			default:
@@ -904,7 +902,7 @@ int pkg_unmerge(char *cat, char *pkgname)
 
 	if (!pretend) {
 		snprintf(buf, sizeof(buf), BUSYBOX " rm -rf %s/%s/%s/%s", portroot, portvdb, cat, pkgname);
-		system(buf);
+		xsystem(buf);
 	}
 	return 1;
 }
@@ -1092,15 +1090,16 @@ void pkg_fetch(int level, depend_atom *atom, struct pkg_t *pkg)
 		fflush(stderr);
 		return;
 	}
-	getcwd(savecwd, sizeof(savecwd));
-	assert(chdir(pkgdir) == 0);
+	xgetcwd(savecwd, sizeof(savecwd));
+	xchdir(pkgdir);
 	if (chdir("All/") == 0) {
 		snprintf(buf, sizeof(buf), "%s.tbz2", pkg->PF);
 		snprintf(str, sizeof(str), "../%s/%s.tbz2", atom->CATEGORY, pkg->PF);
 		unlink(buf);
-		symlink(str, buf);
+		if (symlink(str, buf))
+			errp("symlink(%s, %s) failed", str, buf);
 	}
-	chdir(savecwd);
+	xchdir(savecwd);
 
 	snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
 	if ((pkg_verify_checksums(buf, pkg, atom, qmerge_strict, 1)) == 0) {
@@ -1109,9 +1108,6 @@ void pkg_fetch(int level, depend_atom *atom, struct pkg_t *pkg)
 	}
 }
 
-
-
-
 void print_Pkg(int full, struct pkg_t *pkg)
 {
 	char *p = NULL;
@@ -1215,7 +1211,7 @@ struct pkg_t *grab_binpkg_info(const char *name)
 	snprintf(buf, sizeof(buf), "%s/portage/Packages", port_tmpdir);
 
 	if ((fp = fopen(buf, "r")) == NULL)
-		err("Unable to open package file %s: %s", buf, strerror(errno));
+		errp("Unable to open package file %s", buf);
 
 	while ((fgets(buf, sizeof(buf), fp)) != NULL) {
 		if (*buf == '\n') {
@@ -1321,7 +1317,7 @@ char *find_binpkg(const char *name)
 	snprintf(buf, sizeof(buf), "%s/portage/Packages", port_tmpdir);
 
 	if ((fp = fopen(buf, "r")) == NULL)
-		err("Unable to open package file %s: %s", buf, strerror(errno));
+		errp("Unable to open package file %s", buf);
 
 	while ((fgets(buf, sizeof(buf), fp)) != NULL) {
 		if (*buf == '\n') {
@@ -1398,7 +1394,7 @@ int parse_packages(const char *Packages, int argc, char **argv)
 	long lineno = 0;
 
 	if ((fp = fopen(Packages, "r")) == NULL)
-		err("Unable to open package file %s: %s", Packages, strerror(errno));
+		errp("Unable to open package file %s", Packages);
 
 	memset(&Pkg, 0, sizeof(Pkg));
 
@@ -1497,7 +1493,7 @@ queue *get_world(void) {
 		strncpy(fname,  "/var/lib/portage/world", sizeof(fname));
 
 	if ((fp = fopen(fname, "r")) == NULL) {
-		warn("fopen(\"%s\", \"r\"); = -1 (%s)", fname, strerror(errno));
+		warnp("fopen(\"%s\", \"r\"); = -1", fname);
 		return NULL;
 	}
 
@@ -1559,7 +1555,7 @@ int qmerge_main(int argc, char **argv)
 
 	/* Short circut this. */
 	if ((install || uninstall) && !pretend) {
-		if (getenv("QMERGE") == NULL) {
+		if (follow_rdepends && getenv("QMERGE") == NULL) {
 			uninstall = 0;
 			install = 0;
 			warn("Using these options are likely to break your system at this point. export QMERGE=1; if you think you know what your doing.");
